import re
from flask import current_app
from .util.crest import check_crest_response, NameLookup
from . import static_data


REGION_TYPE = 'application/vnd.ccp.eve.Region-v1+json'
CONSTELLATION_TYPE = 'application/vnd.ccp.eve.Constellation-v1+json'
SYSTEM_TYPE = 'application/vnd.ccp.eve.System-v1+json'

REGION_SLUG = 'https://public-crest.eveonline.com/regions/{}/'
CONSTELLATION_SLUG = 'https://public-crest.eveonline.com/constellations/{}/'
SYSTEM_SLUG = 'https://public-crest.eveonline.com/solarsystems/{}/'


region_names = NameLookup(static_data.region_names, REGION_SLUG, REGION_TYPE)


constellation_names = NameLookup(
        static_data.constellation_names,
        CONSTELLATION_SLUG,
        CONSTELLATION_TYPE)


system_names = NameLookup(static_data.system_names, SYSTEM_SLUG, SYSTEM_TYPE)


class ConstellationRegionLookup(NameLookup):

    def __init__(self):
        super(ConstellationRegionLookup, self).__init__(
                static_data.constellations_to_regions,
                CONSTELLATION_SLUG,
                CONSTELLATION_TYPE,
                'region.href')

    def __getitem__(self, key):
        parent_item = super(ConstellationRegionLookup, self).__getitem__(key)
        # parent item will either be a CREST URL or an integer type ID
        if isinstance(parent_item, int):
            return parent_item
        match = re.match(REGION_SLUG.replace('{}', '(.*)'), parent_item)
        if not match:
            message = "Cannot find the name for the ID requested: {}"\
                    .format(key)
            raise KeyError(message)
        self._dict[key] = int(match.group(1))
        return self._dict[key]


systems_constellations = NameLookup(static_data.systems_to_constellations,
                                    SYSTEM_SLUG,
                                    SYSTEM_TYPE,
                                    'constellation.id')

constellations_regions = ConstellationRegionLookup()



from __future__ import absolute_import
import datetime as dt
from flask import flash, redirect, current_app, url_for, request, session
from flask.ext.babel import gettext
from flask.ext.login import current_user
from requests_oauthlib import OAuth2Session
from oauthlib.oauth2 import OAuth2Error, TokenExpiredError
from flask.ext.wtf import csrf
import six
from sqlalchemy.orm.exc import NoResultFound

from .. import db
from ..util import ensure_unicode, DateTime
from . import AuthMethod
from .models import User, Group, Pilot


class OAuthMethod(AuthMethod):

    def __init__(self, **kwargs):
        """Abstract :py:class:`~.AuthMethod` for OAuth-based login methods.

        Implementing classes need to implement :py:meth:`get_user`\,
        :py:meth:`get_pilots`\, and :py:meth:`get_groups`\.

        In addition to the keyword arguments from :py:class:`~.AuthMethod`,
        this initializer accepts the following arguments that will be used in
        the creation of the :py:attr:`OAuthMethod.oauth` object (See the
        documentation for :py:class:`~flask_oauthlib.client.OAuthRemoteApp` for
        more details):

        * ``client_id``
        * ``client_secret``
        * ``scope``
        * ``access_token_url``
        * ``refresh_token_url``
        * ``authorize_url``
        * ``access_token_params``
        * ``method``

        As a convenience, the ``key`` and ``secret`` keyword arguments will be
        treated as ``consumer_key`` and ``consumer_secret`` respectively. The
        ``name`` argument is used for both :py:class:`~.AuthMethod` and for
        :py:class:`~flask_oauthlib.client.OAuthRemoteApp`.

        Subclasses for providers that may be used by more than one entity are
        encouraged to provide their own defaults for the above arguments.

        The redirect URL for derived classes is based off of the
        :py:attr:`~.AuthMethod.safe_name` of the implementing
        :py:class:`~.AuthMethod`, specifically the URL for
        :py:meth:`~.AuthMethod.view`. For example, the default redirect URL for
        :py:class:`~.TestOAuth` is similar to
        ``https://example.com/login/test_oauth/`` (Note the trailing slash, it
        is significant).
        """
        keyword_mapping = {
            'key': 'client_id',
            'consumer_key': 'client_id',
            'secret': 'client_secret',
            'consumer_secret': 'client_secret',
            'access_token_method': 'method',
        }
        for old_kw, new_kw in six.iteritems(keyword_mapping):
            if old_kw in kwargs:
                # TODO: Raise a deprecation warning
                kwargs[new_kw] = kwargs.pop(old_kw)
        # TODO: add handling for old request_token_params and pulling scope out?
        # Save arguments used for later operations
        self.client_id = kwargs.pop('client_id')
        self.client_secret = kwargs.pop('client_secret')
        self.authorize_url = kwargs.pop('authorize_url')
        self.token_url = kwargs.pop('access_token_url')
        self.refresh_url = kwargs.pop('refresh_token_url', None)
        self.oauth_method = kwargs.pop('method', 'POST')
        self.scope = kwargs.pop('scope', None)
        if 'name' not in kwargs:
            self.name = 'OAuth'
        else:
            self.name = kwargs['name']
        # self.name must be defines before self.safe_name will work
        self.redirect_uri = url_for('login.auth_method_login',
                auth_method=self.safe_name, _external=True)
        try:
            self.access_params = kwargs.pop('access_token_params')
        except KeyError:
            self.access_params = {}
        super(OAuthMethod, self).__init__(**kwargs)

    def login(self, form):
        # CSRF token valid for 5 minutes
        oauth = OAuth2Session(self.client_id,
                redirect_uri=self.redirect_uri,
                scope=self.scope)
        url, state = oauth.authorization_url(self.authorize_url)
        session['state'] = state
        current_app.logger.debug(u"Redirecting to : {}".format(url))
        return redirect(url)

    def view(self):
        """Handle creating and/or logging in the user and updating their
        :py:class:`~.Pilot`\s and :py:class:`~.Group`\s.
        """
        oauth = OAuth2Session(self.client_id,
                redirect_uri=self.redirect_uri,
                state=session['state'])
        try:
            token = oauth.fetch_token(self.token_url,
                    authorization_response=request.url,
                    method=self.oauth_method,
                    client_secret=self.client_secret,
                    auth=(self.client_id, self.client_secret))
        except OAuth2Error as e:
            # TRANS: When there's an error associated with a login.
            flash(gettext(u"Login failed: %(error)s", error=e.error))
            return redirect(url_for('login.login'))
        # Sneaky workaround because current_user isn't set, and self.session
        # relies on current_user
        self._oauth_session = OAuth2Session(self.client_id, token=token)
        # Get the User object for this user, creating one if needed
        user = self.get_user()
        user.token = token[u'access_token']
        if user is not None:
            # Apply site-wide admin flag
            user.admin = self.is_admin(user)
            # Login the user, so current_user will work
            self.login_user(user)
        else:
            # TRANS: Error shown for a failed login.
            flash(gettext(u"Login failed."), u'error')
            return redirect(url_for('login.login'))
        # Add new Pilots
        current_pilots = self.get_pilots()
        for pilot in current_pilots:
            pilot.user = user
        # Remove old pilots
        user_pilots = set(user.pilots)
        for pilot in user_pilots:
            if pilot not in current_pilots:
                pilot.user = None
        # Add new groups
        current_groups = self.get_groups()
        for group in current_groups:
            user.groups.add(group)
        # Remove old groups
        user_groups = set(user.groups)
        for group in user_groups:
            if group not in current_groups and group in user.groups:
                user.groups.remove(group)
        # Save all changes
        db.session.commit()
        return redirect(url_for('index'))

    def get_user(self):
        """Returns the :py:class:`~.OAuthUser` instance for the given token.

        This method is to be implemented by subclasses of
        :py:class:`OAuthMethod` to use whatever APIs they have access to to get
        the user account given an access token.

        :rtype: :py:class:`OAuthUser`
        """
        raise NotImplementedError

    def is_admin(self, user):
        """Returns wether this user should be treated as a site-wide
        administrator.

        The default implementation checks if the user's name is contained
        within the list of administrators supplied as an argument to
        :py:class:`OAuthMethod`\.

        :param user: The user to check.
        :type user: :py:class:`~.OAuthUser`
        :rtype: bool
        """
        return user.name in self.admins

    def get_pilots(self):
        """Return a :py:class:`list` of :py:class:`~.Pilot`\s for the given
        token.

        Like :py:meth:`get_user`\, this method is to be implemented by
        :py:class:`OAuthMethod` subclasses to return a list of
        :py:class:`~.Pilot`\s associated with the account for the given access
        token.

        :rtype: :py:class:`list` of :py:class:`~.Pilot`\s.
        """
        raise NotImplementedError

    def get_groups(self):
        """Returns a :py:class:`list` of :py:class:`~.Group`\s for the given
        token.

        Like :py:meth:`get_user` and :py:meth:`get_pilots`\, this method is to
        be implemented by :py:class:`OAuthMethod` subclasses to return a list
        of :py:class:`~.Group`\s associated with the account for the given
        access token.

        :rtype: :py:class:`list` of :py:class:`~.Group`\s.
        """
        raise NotImplementedError

    @property
    def session(self):
        if not hasattr(self, '_oauth_session'):
            if not current_user.is_anonymous:
                token = {'token_type': 'Bearer'}
                token['access_token'] = current_user.token
                kwargs['token'] = token
                self._oauth_session = OAuth2Session(self.client_id, **kwargs)
        return self._oauth_session


class OAuthUser(User):

    id = db.Column(db.Integer, db.ForeignKey(User.id), primary_key=True)

    token = db.Column(db.String(100, convert_unicode=True))



#!/usr/bin/env python
from setuptools import setup
import re
import sys


with open('evesrp/__init__.py', 'r') as f:
    init_contents = ''
    for line in f:
        init_contents += line + '\n'
version = re.search(r'^__version__ *= *u?[\'"]([^\'"]*)[\'"]', init_contents,
        re.MULTILINE)
if version:
    version = version.group(1)
else:
    raise Exception(u"Unable to find __version__ in evesrp/__init__.py")


test_requirements = [
    'beautifulsoup>=4.4.1',
    'coverage>=3.7.1',
    'httmock>=1.2.2',
]
# unittest.mock was added in 3.3, but is available as a backport as the 'mock'
# package on PyPI.
if sys.version_info.major < 3 or sys.version_info.minor < 3:
    test_requirements.append('mock>=1.0.1')


setup(
    name=u'EVE-SRP',
    version=version,
    description=u'EVE Ship Replacement Program Helper',
    author=u'Will Ross',
    author_email=u'paxswill@paxswill.com',
    url=u'https://github.com/paxswill/evesrp',
    packages=[
        'evesrp',
        'evesrp.auth',
        'evesrp.views',
        'evesrp.util',
        'evesrp.migrate',
        'evesrp.migrate.versions',
    ],
    package_data={
        'evesrp': [
            'static/css/*.css',
            'static/css/*.css.map',
            'static/evesso.png',
            'static/favicon.ico',
            'static/fonts/*.eot',
            'static/fonts/*.ttf',
            'static/fonts/*.svg',
            'static/fonts/*.woff',
            'static/fonts/*.woff2',
            'static/fonts/*.otf',
            'static/js/evesrp.min.js',
            'static/js/evesrp.min.js.map',
            'static/js/formatters.min.js',
            'static/js/formatters.min.js.map',
            'static/translations/*.json',
            'static/ZeroClipboard.swf',
            'templates/*.html',
            'templates/*.xml',
            'translations/*/LC_MESSAGES/messages.*',
            'migrate/alembic.ini',
            'migrate/script.py.mako',
        ],
    },
    classifiers=[
        u'Development Status :: 4 - Beta',
        u'Framework :: Flask',
        u'License :: OSI Approved :: BSD License',
        u'Programming Language :: Python :: 3',
        u'Programming Language :: Python :: 2',
        u'Topic :: Games/Entertainment',
    ],
    install_requires=[
        'Babel>=2.2.0',
        'Flask>=0.10.1',
        'Flask-Babel>=0.9',
        'Flask-Login>=0.3.2',
        'Flask-Migrate>=1.2.0',
        'Flask-Script==2.0.5',
        'Flask-SQLAlchemy==2.0',
        'Flask-WTF==0.10.2',
        'SQLAlchemy>=0.9.7',
        'Werkzeug>=0.9.4',
        'WTForms>=2.0.0',
        'alembic>=0.6.5',
        'requests>=2.9.1',
        'six==1.7.3',
        'iso8601>=0.1.5',
    ],
    test_suite='tests',
    test_require=test_requirements,
    entry_points={
        'console_scripts': [
            'evesrp = evesrp.util.manage:main',
        ],
    },
    extras_require={
        'BraveCore': [
            'braveapi==0.1',
            'ecdsa==0.11',
        ],
        'OAuth': [
            'requests-oauthlib>=0.6.0',
        ],
        'Sentry': [
            'raven[flask]>=5.10.2',
        ],
    },
    zip_safe=False,
)



from __future__ import absolute_import
from __future__ import unicode_literals
from flask import redirect, url_for, render_template, make_response, request,\
        json, session
from flask.ext import babel as flask_babel
from flask.ext.login import login_required, current_user
from babel import get_locale_identifier, negotiate_locale, parse_locale
import six
from .. import db, babel
from ..models import Request, ActionType
from ..auth import PermissionType
from ..auth.models import Permission, Division
from ..util import jsonify, varies, locale


if six.PY3:
    unicode = str


@login_required
def index():
    """The index page for EVE-SRP."""
    return redirect(url_for('requests.personal_requests'))


@varies('Accept', 'X-Requested-With')
def error_page(error):
    """View function for displaying error pages."""
    # Try to get some meaningful bits of information about the error
    # HTTPExceptions (raised by abort()) have code and description attributes.
    # Try to prefer those rich values over more generic information.
    code = error.code if hasattr(error, 'code') else 500
    if hasattr(error, 'description'):
        description = error.description
    else:
        description = str(error)
    name = error.name if hasattr(error, 'name') else u'Application Error'
    # Give the error information in a machine readable format for APIs
    if request.is_json or request.is_xhr:
        response_content = jsonify(description=error.description,
                code=code)
    elif request.is_xml:
        pass
    else:
        response_content = render_template('error.html', code=code,
                description=description, name=name, title=code)
    # Give a default response code for generic exceptions
    return make_response(response_content, code)


def request_count(permission, statuses=None):
    """Function intended for counting the number of requests for Jinja
    templates.
    """
    if statuses is None:
        if permission == PermissionType.review:
            statuses = (ActionType.evaluating,)
        elif permission == PermissionType.pay:
            statuses = (ActionType.approved,)
        elif permission == PermissionType.submit:
            statuses = (ActionType.incomplete,)
        else:
            return 0
    elif statuses in ActionType.statuses:
        statuses = (statuses,)
    permissions = current_user.permissions.\
            filter(Permission.permission==permission).\
            subquery()
    divisions = db.session.query(Division.id).\
            join(permissions).\
            subquery()
    requests = db.session.query(db.func.count(db.distinct(Request.id))).\
            join(divisions).\
            filter(Request.status.in_(statuses))
    if permission == PermissionType.submit:
        requests = requests.filter(Request.submitter==current_user)
    return requests.one()[0]


def update_navbar(response):
    if request.endpoint == 'static':
        return response
    if 'application/json' not in response.mimetype:
        return response
    response_json = json.loads(response.get_data())
    counts = {
        'pending': 0,
        'payouts': 0,
        'personal': 0
    }
    response_json['nav_counts'] = counts
    # Unauthenticated users get nothing
    if not current_user.is_authenticated:
        response.set_data(json.dumps(response_json))
        return response
    counts['pending'] = request_count(PermissionType.review)
    counts['payouts'] = request_count(PermissionType.pay)
    counts['personal'] = request_count(PermissionType.submit)
    response.set_data(json.dumps(response_json))
    return response


def detect_language():
    if 'lang' in request.args:
        requested_locale = request.args['lang']
        locales = [unicode(l) for l in babel.list_translations()]
        locale = negotiate_locale([requested_locale,], locales)
        session['locale'] = locale
        flask_babel.refresh()


def locale_selector():
    requested_locale = session.get('locale')
    supported_locales = [unicode(l) for l in locale.enabled_locales()]
    if requested_locale is not None and \
            requested_locale not in supported_locales:
        requested_locale = None
        del session['locale']
    return requested_locale



from __future__ import unicode_literals

from .util.crest import NameLookup
from . import static_data




ships = NameLookup(static_data.ships,
        'https://public-crest.eveonline.com/types/{}/',
        'application/vnd.ccp.eve.ItemType-v3+json')
